open DataTypes;	
%%
%name BuildAst

%term LPAREN
	| RPAREN
	| NEW
	| NUM of int
	| REAL of real 
	| STRING of string
	| IDENTIFIER of string 
	| PLUS | MINUS
	| MULT | DIV | MOD
	| AND | OR | NEG
	| TRUE
	| FALSE
	| ASSIGN
	| IMPL
	| RIMPL
	| EQUALS
	| LESS
	| MORE
	| LESSEQ
	| MOREEQ
	| EOF

%nonterm Line of exp
	| Exp of exp
	| Aexp of exp
	| Imp of exp
	| Sexp of exp
	| Term of exp
	| Factor of exp
	| Bexp of exp
	| Beq of exp
	| Bimp of exp
	| Bterm of exp
	| Bfact of exp
	| Sterm of exp
	| Iden of exp
	| AOP of exp*exp -> exp
	| MOP of exp*exp -> exp
	| REL of exp*exp -> exp
	| BOOL of exp

%pos int
%eop EOF
%noshift EOF
%nodefault
%verbose
%arg (fileName) : string

%%
Line : Exp NEW				(STMT(Exp))
	|Exp NEW Line			(SEQ(STMT(Exp),Line))
	|Exp 				(STMT(Exp))
Exp : Aexp				(Aexp)
	|IDENTIFIER ASSIGN Exp		(ASMT(IDENTIFIER,Exp))

Aexp : Imp IMPL Aexp			(IMP(Imp,Aexp))
	|Imp RIMPL Aexp			(IMP(Aexp,Imp))
	|Imp EQUALS Aexp		(EQ(Imp,Aexp))
	|Imp				(Imp)
Imp : Term AOP Imp			(AOP(Term,Imp))
	|Term				(Term)
AOP :  PLUS				(PLUS)
	|MINUS				(MINUS)
	|OR				(OR)
Term : Factor MOP Term			(MOP(Factor,Term))
	|Factor				(Factor)	
MOP :  MULT				(MULT)
	|DIV				(DIV)
	|MOD				(MOD)
	|AND				(AND)
Factor : LPAREN Aexp RPAREN		(Aexp)
	| NUM				(N(NUM))
	| REAL				(R(REAL))
	| TRUE				(B(true))
	| FALSE				(B(false))
	| STRING			(S(STRING))
	| IDENTIFIER			(V(IDENTIFIER))
	| NEG Factor			(NOT(Factor))
	| Aexp REL Aexp			(REL(Aexp1,Aexp2))

REL : LESS				(LT)
	|MORE				(GT)
	|LESSEQ				(LTE)
	|MOREEQ				(GTE)

